@@ -2,16 +2,17 @@
   (:require [schema.core :as schema]
             [puppetlabs.services.jruby.jruby-puppet-schemas :as jruby-schemas]
             [puppetlabs.services.jruby.puppet-environments :as puppet-env]
-            [me.raynes.fs :as fs]
-            [clojure.tools.logging :as log])
+            [clojure.tools.logging :as log]
+            [puppetlabs.kitchensink.core :as ks])
   (:import (com.puppetlabs.puppetserver PuppetProfiler JRubyPuppet)
            (com.puppetlabs.puppetserver.pool JRubyPool)
-           (puppetlabs.services.jruby.jruby_puppet_schemas JRubyPuppetInstance PoisonPill)
+           (puppetlabs.services.jruby.jruby_puppet_schemas JRubyPuppetInstance PoisonPill ShutdownPoisonPill)
            (java.util HashMap)
            (org.jruby CompatVersion Main RubyInstanceConfig RubyInstanceConfig$CompileMode)
-           (org.jruby.embed ScriptingContainer LocalContextScope)
+           (org.jruby.embed LocalContextScope)
            (java.util.concurrent TimeUnit)
-           (clojure.lang IFn)))
+           (clojure.lang IFn)
+           (com.puppetlabs.puppetserver.jruby ScriptingContainer)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Definitions
@@ -29,11 +30,6 @@
   See also:  http://jruby.org/apidocs/org/jruby/runtime/load/LoadService.html"
   "puppet-server-lib")
 
-(def compile-mode
-  "The JRuby compile mode to use for all ruby components, e.g. the master
-  service and CLI tools."
-  RubyInstanceConfig$CompileMode/OFF)
-
 (def compat-version
   "The JRuby compatibility version to use for all ruby components, e.g. the
   master service and CLI tools."
@@ -46,6 +42,7 @@
   (schema/pred (some-fn nil?
                  jruby-schemas/poison-pill?
                  jruby-schemas/retry-poison-pill?
+                 jruby-schemas/shutdown-poison-pill?
                  jruby-schemas/jruby-puppet-instance?)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -95,44 +92,55 @@
   [ruby-load-path :- [schema/Str]]
   (cons ruby-code-dir ruby-load-path))
 
-(defn prep-scripting-container
-  [scripting-container ruby-load-path gem-home]
-  ; Note, this behavior should remain consistent with new-main
-  (doto scripting-container
+(schema/defn ^:always-validate get-compile-mode :- RubyInstanceConfig$CompileMode
+  [config-compile-mode :- jruby-schemas/SupportedJRubyCompileModes]
+  (case config-compile-mode
+    :jit RubyInstanceConfig$CompileMode/JIT
+    :force RubyInstanceConfig$CompileMode/FORCE
+    :off RubyInstanceConfig$CompileMode/OFF))
+
+(schema/defn ^:always-validate init-jruby-config :- jruby-schemas/ConfigurableJRuby
+  "Applies configuration to a JRuby... thing.  See comments in `ConfigurableJRuby`
+  schema for more details."
+  [jruby-config :- jruby-schemas/ConfigurableJRuby
+   ruby-load-path :- [schema/Str]
+   gem-home :- schema/Str
+   compile-mode :- jruby-schemas/SupportedJRubyCompileModes]
+  (doto jruby-config
     (.setLoadPaths (managed-load-path ruby-load-path))
     (.setCompatVersion compat-version)
-    (.setCompileMode compile-mode)
+    (.setCompileMode (get-compile-mode compile-mode))
     (.setEnvironment (managed-environment (get-system-env) gem-home))))
 
-(defn empty-scripting-container
-  "Creates a clean instance of `org.jruby.embed.ScriptingContainer` with no code loaded."
-  [ruby-load-path gem-home]
+(schema/defn ^:always-validate empty-scripting-container :- ScriptingContainer
+  "Creates a clean instance of a JRuby `ScriptingContainer` with no code loaded."
+  [ruby-load-path :- [schema/Str]
+   gem-home :- schema/Str
+   compile-mode :- jruby-schemas/SupportedJRubyCompileModes]
   {:pre [(sequential? ruby-load-path)
          (every? string? ruby-load-path)
          (string? gem-home)]
    :post [(instance? ScriptingContainer %)]}
   (-> (ScriptingContainer. LocalContextScope/SINGLETHREAD)
-    (prep-scripting-container ruby-load-path gem-home)))
+      (init-jruby-config ruby-load-path gem-home compile-mode)))
 
-(defn create-scripting-container
+(schema/defn ^:always-validate create-scripting-container :- ScriptingContainer
   "Creates an instance of `org.jruby.embed.ScriptingContainer` and loads up the
   puppet and facter code inside it."
-  [ruby-load-path gem-home]
-  {:pre [(sequential? ruby-load-path)
-         (every? string? ruby-load-path)
-         (string? gem-home)]
-   :post [(instance? ScriptingContainer %)]}
+  [ruby-load-path :- [schema/Str]
+   gem-home :- schema/Str
+   compile-mode :- jruby-schemas/SupportedJRubyCompileModes]
   ;; for information on other legal values for `LocalContextScope`, there
   ;; is some documentation available in the JRuby source code; e.g.:
   ;; https://github.com/jruby/jruby/blob/1.7.11/core/src/main/java/org/jruby/embed/LocalContextScope.java#L58
   ;; I'm convinced that this is the safest and most reasonable value
   ;; to use here, but we could potentially explore optimizations in the future.
-  (doto (empty-scripting-container ruby-load-path gem-home)
+  (doto (empty-scripting-container ruby-load-path gem-home compile-mode)
     ;; As of JRuby 1.7.20 (and the associated 'jruby-openssl' it pulls in),
     ;; we need to explicitly require 'jar-dependencies' so that it is used
     ;; to manage jar loading.  We do this so that we can instruct
     ;; 'jar-dependencies' to not actually load any jars.  See the environment
-    ;; variable configuration in 'prep-scripting-container' for more
+    ;; variable configuration in 'init-jruby-config' for more
     ;; information.
     (.runScriptlet "require 'jar-dependencies'")
     (.runScriptlet "require 'puppet/server/master'")))
@@ -148,7 +156,7 @@
                            [:master-run-dir "rundir"]
                            [:master-log-dir "logdir"]]]
       (if-let [value (get config setting)]
-        (.put puppet-config dir (fs/absolute-path value))))
+        (.put puppet-config dir (ks/absolute-path value))))
     puppet-config))
 
 (schema/defn borrow-with-timeout-fn :- JRubyPuppetInternalBorrowResult
@@ -166,6 +174,15 @@
   {:pool (instantiate-free-pool size)
    :size size})
 
+(schema/defn ^:always-validate
+  cleanup-pool-instance!
+  "Cleans up and cleanly terminates a JRubyPuppet instance and removes it from the pool."
+  [{:keys [scripting-container jruby-puppet pool] :as instance} :- JRubyPuppetInstance]
+  (.unregister pool instance)
+  (.terminate jruby-puppet)
+  (.terminate scripting-container)
+  (log/infof "Cleaned up old JRuby instance with id %s." (:id instance)))
+
 (schema/defn ^:always-validate
   create-pool-instance! :- JRubyPuppetInstance
   "Creates a new JRubyPuppet instance and adds it to the pool."
@@ -174,7 +191,7 @@
    config   :- jruby-schemas/JRubyPuppetConfig
    flush-instance-fn :- IFn
    profiler :- (schema/maybe PuppetProfiler)]
-  (let [{:keys [ruby-load-path gem-home
+  (let [{:keys [ruby-load-path gem-home compile-mode
                 http-client-ssl-protocols http-client-cipher-suites
                 http-client-connect-timeout-milliseconds
                 http-client-idle-timeout-milliseconds
@@ -182,7 +199,11 @@
     (when-not ruby-load-path
       (throw (Exception.
                "JRuby service missing config value 'ruby-load-path'")))
-    (let [scripting-container   (create-scripting-container ruby-load-path gem-home)
+    (log/infof "Creating JRuby instance with id %s." id)
+    (let [scripting-container   (create-scripting-container
+                                 ruby-load-path
+                                 gem-home
+                                 compile-mode)
           env-registry          (puppet-env/environment-registry)
           ruby-puppet-class     (.runScriptlet scripting-container "Puppet::Server::Master")
           puppet-config         (config->puppet-config config)
@@ -204,12 +225,13 @@
                         :max-requests         (:max-requests-per-instance config)
                         :flush-instance-fn    flush-instance-fn
                         :state                (atom {:borrow-count 0})
-                        :jruby-puppet         (.callMethod scripting-container
-                                                           ruby-puppet-class
-                                                           "new"
-                                                           (into-array Object
-                                                                       [puppet-config puppet-server-config])
-                                                           JRubyPuppet)
+                        :jruby-puppet         (.callMethodWithArgArray
+                                               scripting-container
+                                               ruby-puppet-class
+                                               "new"
+                                               (into-array Object
+                                                           [puppet-config puppet-server-config])
+                                               JRubyPuppet)
                         :scripting-container  scripting-container
                         :environment-registry env-registry})]
         (.register pool instance)
@@ -258,12 +280,15 @@
           ((some-fn nil? jruby-schemas/retry-poison-pill?) instance)
           instance
 
+          (instance? ShutdownPoisonPill instance)
+          instance
+
           :else
           (throw (IllegalStateException.
                    (str "Borrowed unrecognized object from pool!: " instance))))))
 
 (schema/defn ^:always-validate
-  borrow-from-pool :- jruby-schemas/JRubyPuppetInstanceOrRetry
+  borrow-from-pool :- jruby-schemas/JRubyPuppetInstanceOrPill
   "Borrows a JRubyPuppet interpreter from the pool. If there are no instances
   left in the pool then this function will block until there is one available."
   [pool-context :- jruby-schemas/PoolContext]
@@ -287,7 +312,7 @@
 (schema/defn ^:always-validate
   return-to-pool
   "Return a borrowed pool instance to its free pool."
-  [instance :- jruby-schemas/JRubyPuppetInstanceOrRetry]
+  [instance :- jruby-schemas/JRubyPuppetInstanceOrPill]
   (if (jruby-schemas/jruby-puppet-instance? instance)
     (let [new-state (swap! (:state instance)
                            update-in [:borrow-count] inc)
@@ -313,12 +338,10 @@
   e.g. for the ruby, gem, and irb subcommands.  Internal core services should
   use `create-scripting-container` instead of `new-main`."
   [config :- jruby-schemas/JRubyPuppetConfig]
-  (let [jruby-config (RubyInstanceConfig.)
-        {:keys [ruby-load-path gem-home]} config]
-    ; Note, this behavior should remain consistent with prep-scripting-container
-    (doto jruby-config
-      (.setLoadPaths (managed-load-path ruby-load-path))
-      (.setCompatVersion compat-version)
-      (.setCompileMode compile-mode)
-      (.setEnvironment (managed-environment (get-system-env) gem-home)))
+  (let [{:keys [ruby-load-path gem-home compile-mode]} config
+        jruby-config (init-jruby-config
+                      (RubyInstanceConfig.)
+                      ruby-load-path
+                      gem-home
+                      compile-mode)]
     (Main. jruby-config)))
